@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Johan Dahlin
+# Copyright (C) 2005 Johan Dahlin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public License
@@ -130,10 +130,17 @@ class IObjectAdapter:
 class Adapter(object):
     __metaclass__ = AdapterMeta
     _adapters = []
+    _properties = {}
 
     def __init__(self, build):
         self._build = build
-        
+
+    def get_raw_property(self, object, name):
+        object_id = object.get_data('gazpacho::object-id')
+        if not object_id in self._properties:
+            raise KeyError(object_id)
+        return self._properties[object_id][name]
+    
     def add_adapter(cls):
         if cls.__name__ != 'Adapter':
             cls._adapters.append(cls)
@@ -151,6 +158,9 @@ class AdapterRegistry:
         for adapter in Adapter.get_adapters():
             self.register_adapter(adapter)
              
+    def _add_adapter(self, name, adapter):
+        self._adapters[name] = adapter
+
     def register_adapter(self, adapter):
         adict = adapter.__dict__
         # Do not register abstract adapters
@@ -160,12 +170,10 @@ class AdapterRegistry:
                 object_type = object_type,
  
             for klass in object_type:
-                type_name = gobject.type_name(klass)
-                self._adapters[type_name] = adapter
+                self._add_adapter(gobject.type_name(klass), adapter)
         elif adict.has_key('object_name'):
-            object_name = adict.get('object_name')
-            self._adapters[object_name] = adapter
-
+            self._add_adapter(adict.get('object_name'), adapter)
+            
     def get_adapter(self, gobj, build=None):
         orig = gobj
         while True:
@@ -202,12 +210,18 @@ class GObjectAdapter(Adapter):
             prop_names.append(name)
 
         gobj.set_data('gobject.changed_properties', prop_names)
-    
+
     def _getproperties(self, gtype, obj_id, properties, child=False):
         if child:
             get_pspec = get_child_pspec_from_name
         else:
             get_pspec = get_pspec_from_name
+
+        propdict = self._properties
+        if not propdict.has_key(obj_id):
+            props = propdict[obj_id] = {}
+        else:
+            props = propdict[obj_id]
             
         construct = {}
         normal = []
@@ -215,6 +229,7 @@ class GObjectAdapter(Adapter):
         for prop in properties:
             propname = prop.name.replace('_', '-')
             #full = '%s::%s' % (parent_name, propname)
+            props[prop.name] = prop.data
             if hasattr(self, 'prop_set_%s' % prop.name):
                 normal.append((prop.name, prop.data))
                 continue
@@ -251,7 +266,7 @@ class GObjectAdapter(Adapter):
         if child:
             assert not construct
             return normal
-        
+
         return construct, normal
 
     def _valuefromstring(self, obj_id, pspec, string):
@@ -305,6 +320,9 @@ class GObjectAdapter(Adapter):
             raise AssertionError("type %r is unknown" % prop_type)
 
         return value
+
+    def add(self, parent, child, properties):
+        raise NotImplementedError
     
     def find_internal_child(self, gobj, name):
         if name in self.child_names:
@@ -328,6 +346,7 @@ class WidgetAdapter(GObjectAdapter):
         return widget
     
     def prop_set_has_default(self, widget, value):
+        value = str2bool(value)
         if value:
             self._build._default_widget = widget
 
@@ -338,15 +357,24 @@ class WidgetAdapter(GObjectAdapter):
         else:
             self._build._tooltips.set_tip(widget, value, None)
 
-    def prop_set_has_focus(self, widget, value):
-        if value:
-            self._build._focus_widget = widget
-    
     def prop_set_visible(self, widget, value):
         value = str2bool(value)
         widget.set_data('gazpacho::visible', value)
         widget.set_property('visible', value)
-                           
+
+    def prop_set_has_focus(self, widget, value):
+        value = str2bool(value)
+        if value:
+            self._build._focus_widget = widget
+        widget.set_data('gazpacho::has-focus', value)
+    
+    def prop_set_is_focus(self, widget, value):
+        value = str2bool(value)
+        widget.set_data('gazpacho::is-focus', value)
+
+    def prop_set_sizegroup(self, widget, value):
+        widget.set_data('gazpacho::sizegroup', value)
+        
 class PythonWidgetAdapter(WidgetAdapter):
     def construct(self, name, gtype, properties):
         obj = self.object_type()
@@ -492,7 +520,11 @@ class WindowAdapter(ContainerAdapter):
         
     def prop_set_wmclass_name(self, window, value):
         window.set_wmclass(value, window.wmclass_name)
-    
+   
+    def prop_set_type_hint(self, window, value):
+        window.set_data('gazpacho::type-hint', 
+                        enumfromstring(value, enum=gdk.WindowTypeHint))
+
 class NotebookAdapter(ContainerAdapter):
     object_type = gtk.Notebook
     def add(self, notebook, child, properties):
@@ -665,12 +697,16 @@ class ToolButtonAdapter(ContainerAdapter):
     object_type = gtk.ToolButton
     def prop_set_icon(self, toolbutton, value):
         filename = self._build.find_resource(value)
+        if not filename:
+            print 'No such a file or directory: %s' % value
+            return
+        
         pb = gdk.pixbuf_new_from_file(filename)
         if not pb:
             print "warning: Couldn't find image file: %s" %  value
             return
-
-        image = gtk.image_new_from_pixbuf(pb)
+        image = gtk.Image()
+        image.set_from_pixbuf(pb)
         image.show()
         toolbutton.set_icon_widget(image)
 
@@ -691,7 +727,8 @@ class ComboBoxAdapter(ContainerAdapter):
         # GtkComboBoxEntry creates the cell renderer itself, but we have to set
         # the column containing the text.
         if isinstance(combobox, gtk.ComboBoxEntry):
-            combobox.set_text_column(0)
+            if combobox.get_text_column() == -1:
+                combobox.set_text_column(0)
         else:
             cell = gtk.CellRendererText()
             combobox.pack_start(cell, True)
@@ -700,6 +737,16 @@ class ComboBoxAdapter(ContainerAdapter):
         for part in value.split('\n'):
             store.append([part])
 
+	    active = combobox.get_data('gazpacho::active')
+	    if active:
+	        combobox.set_active(active)
+
+    def prop_set_active(self, combobox, value):
+        combobox.set_data('gazpacho::active', int(value))
+
+        # We don't need to set the actual property here, it's done in 
+        # prop_set_items.
+
 # FIXME: PyGTK does not expose vscrollbar and hscrollbar.
 #
 #class ScrolledWindowAdapter(WindowAdapter):
@@ -726,7 +773,7 @@ class ColorSelectionDialogAdapter(DialogAdapter):
     child_names = DialogAdapter.child_names + ['ok_button',
                                                'cancel_button',
                                                'help_button',
-                                               'color_selection']
+                                               'colorsel']
     object_type = gtk.ColorSelectionDialog
     def find_internal_child(self, gobj, name):
         if name == 'color_selection':
@@ -744,7 +791,7 @@ class FontSelectionDialogAdapter(DialogAdapter):
     child_names = DialogAdapter.child_names + ['ok_button',
                                                'cancel_button',
                                                'apply_button',
-                                               'color_selection']
+                                               'fontsel']
     def find_internal_child(self, gobj, name):
         if name == 'font_selection':
             return gobj.fontsel
@@ -793,6 +840,23 @@ class TreeViewColumnAdapter(GObjectAdapter):
         else:
             column.pack_end(child, expand)
 
+# Gross hack to make it possible to use FileChooserDialog on win32.
+# See bug http://bugzilla.gnome.org/show_bug.cgi?id=314527
+class FileChooserDialogHack(gtk.FileChooserDialog):
+    def get_children(self):
+        return []
+
+class FileChooserDialogAdapter(DialogAdapter):
+    gobject.type_register(FileChooserDialogHack)
+
+class ImageAdapter(WidgetAdapter):
+    object_type = gtk.Image
+    def prop_set_file(self, image, value):
+        newvalue = self._build.find_resource(value)
+        if newvalue:
+            image.set_property('file', newvalue)
+        image.set_data('image-file-name', value)
+        
 # Global registry
 adapter_registry = AdapterRegistry()
 
