@@ -1,9 +1,29 @@
+# Copyright (C) 2005 by Johan Dahlin
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+# properties.py - More black magic than Dumbledore can handle
+
 import gettext
 
 import gobject
 import gtk
 
 from gazpacho.choice import enum_to_string, flags_to_string
+from gazpacho.environ import environ
+from gazpacho.loader.custom import adapter_registry
 
 _ = gettext.gettext
 
@@ -12,6 +32,40 @@ UNICHAR_PROPERTIES = ('invisible-char', )
 class PropertyError(Exception):
     pass
 
+class UnsupportedProperty(PropertyError):
+    pass
+
+class PropertyCustomEditor(object):
+    """
+    Base class for editors for custom properties
+
+    The create method is called the first time a widget with this property is
+    shown in the editor. Then, everytime a instance of this type of widget is
+    selected the 'update' method is called with the proper information to
+    manipulate the widget.
+
+    Because of that this should not store any information about the state of
+    the widget that is being edited.
+    """
+
+    # if wide_editor is True the editor will expand both columns of the
+    # editor page
+    wide_editor = False
+
+    def get_editor_widget(self):
+        "Return the gtk widget that implements this editor"
+        
+    def update(self, context, gtk_widget, proxy):
+        """
+        Called when a widget with this property is selected
+
+        @param    context: gazpacho context to get the current app and project
+        @param gtk_widget: the widget we are editing
+        @param      proxy: an object that should be used to update the widget.
+                           This is what allows the undo/redo mechanism actually
+                           works
+        """
+
 class PropRegistry:
     def __init__(self):
         self._prop_types = {}
@@ -31,7 +85,7 @@ class PropRegistry:
             raise PropertyError("base type %r is already defined" % gtype)
         elif klass.base_type != None:
             raise PropertyError("klass type %r is already registered to %r" %
-                                klass)
+                                (klass, klass.base_type))
         
         self._bases[gtype] = klass
         klass.base_type = gtype
@@ -40,10 +94,10 @@ class PropRegistry:
         """
         Create a property for full_name, (type::prop_name)
         If it doesn't exist add a custom property
-        @param klass property class
-        @type  klass subclass of PropType
-        @param full_name complete name of property 
-        @type  full_name string (typename::prop_name)
+        @param klass: property class
+        @type  klass: subclass of PropType
+        @param full_name: complete name of property 
+        @type  full_name: string (typename::prop_name)
         """
 
         if not issubclass(klass, PropType):
@@ -56,38 +110,53 @@ class PropRegistry:
         if not self._has_property(type_name, prop_name):
             self._add_custom_type(klass, type_name, prop_name, pspec)
         else:
+            # XXX: Kill this, needs to fix readable/writable first
+            #raise TypeError("%s exists, use override_simple" % full_name)
             self._add_type(full_name, klass, pspec)
-
+        
         # Note, that at this point we need to setup the overriden class
         # since we're not going to call the type constructor, so
         # this needs to be in sync with PropMeta.new
 
         klass.name = prop_name
+        klass.owner_name = type_name
         klass.owner_type = gobject.type_from_name(type_name)
-        klass.child = child
+        
+        # Do not always override, so a base type can specify it
+        if klass.child is None:
+            klass.child = child
+        
+        if not issubclass(klass.editor, PropertyCustomEditor):
+            raise TypeError("editor %r for property class %r needs to be a "
+                            "subclass of PropertyCustomEditor " % (
+                klass.editor, klass))
 
-    def override_simple(self, full_name, bases=(), **kwargs):
+    def override_simple(self, full_name, base=None, child=False, **kwargs):
         """
-        Sample as override_property, but creates and registeres the class for you
-        @param full_name complete name of property
-        @param bases tuple of bases
+        Same as override_property, but creates and registeres the class
+        @param full_name: complete name of property
+        @param base:      base class
+        @param child:     child property
         @param kwargs namespace to use in class
         """
 
-        if not bases:
-            bases = CustomProperty,
-        elif type(bases) != tuple:
-            bases = bases,
-            
         type_name, prop_name = full_name.split('::')
-        pspec = self._get_pspec(type_name, prop_name)
+        parent = None
+        if child or (base and base.child):
+            parent = type_name
+            
+        pspec = self._get_pspec(type_name, prop_name, parent)
         if not pspec:
             raise TypeError("You can only override existing properties, %s "
                             "is not a property" % full_name)
 
-        bases += self._get_base_type(pspec.value_type),
-        klass = type(full_name, bases, kwargs)
-        self.override_property(full_name, klass)
+        if not base:
+            base = CustomProperty
+            
+        parent = self._get_base_type(pspec.value_type)
+        klass = type(full_name, (base, parent), kwargs)
+
+        return self._create_type_from_pspec(pspec, klass, child=child)
         
     def list(self, type_name, parent=None):
         """
@@ -95,10 +164,10 @@ class PropRegistry:
         If parent is specified, child properties will also be included.
         This method will create the prop_types and save if they're
         not already created.
-        @param type_name name of type
-        @type  type_name string
-        @param parent    parent type of object
-        @type  parent    GType
+        @param type_name: name of type
+        @type  type_name: string
+        @param parent:    parent type of object
+        @type  parent:    GType
         """
 
         property_types = []
@@ -106,12 +175,17 @@ class PropRegistry:
             property_types.append(self._get_type_from_pspec(pspec))
 
         if parent:
-            property_types.extend(self._list_child_properties(type_name,
-                                                              parent))
+            prop_types = []
+            for pspec in self._list_child_properties(parent):
+                prop_types.append(self._get_type_from_pspec(pspec, child=True))
+            property_types.extend(prop_types)
+            
         property_types.extend(self._list_custom(type_name))
         
         for prop_type in property_types[:]:
-            if (not prop_type.readable or not prop_type.writable):
+            if (not prop_type.readable or
+                not prop_type.writable or
+                prop_type.disabled):
                 property_types.remove(prop_type)
 
         return property_types
@@ -120,10 +194,10 @@ class PropRegistry:
         """
         Fetch a property from type_name called prop_name, note that it
         does not create the property type if it's not already created.
-        @param type_name gtype name of owner
-        @type  type_name string
-        @param prop_name name of property 
-        @type  prop_name string
+        @param type_name: gtype name of owner
+        @type  type_name: string
+        @param prop_name: name of property 
+        @type  prop_name: string
         """
         pspec = self._get_pspec(type_name, prop_name, parent)
         if not pspec:
@@ -134,7 +208,7 @@ class PropRegistry:
             return prop_type
 
         child = False
-        if pspec in self._list_child_properties(type_name, parent):
+        if pspec in self._list_child_properties(parent):
             child = True
                                                 
         return self._get_type_from_pspec(pspec, child=child)
@@ -163,8 +237,9 @@ class PropRegistry:
             raise PropertyError("Can't find base type for %r" % base_type)
         return base
         
-    def _create_type_from_pspec(self, pspec, child=False):
-        base = self._get_base_type(pspec.value_type)
+    def _create_type_from_pspec(self, pspec, base=None, child=False):
+        if not base:
+            base = self._get_base_type(pspec.value_type)
         
         readable = pspec.flags & gobject.PARAM_READABLE != 0
         writable = pspec.flags & gobject.PARAM_WRITABLE != 0
@@ -174,7 +249,8 @@ class PropRegistry:
                                  pspec.owner_type,
                                  readable=readable,
                                  writable=writable,
-                                 child=child)
+                                 child=child,
+                                 label=pspec.nick)
         self._add_type(prop_type.__name__, prop_type, pspec)
         return prop_type
 
@@ -188,10 +264,16 @@ class PropRegistry:
             prop_type.label = pspec.nick
             prop_type.type = pspec.value_type
         else:
-            prop_type.label = prop_name.capitalize()
-        #
+            if not prop_type.label:
+                prop_type.label = prop_name.capitalize()
+
+        # We can't enable this just yet. child properties
+        # are overriding existing ones, they need a another storage
+        #if prop_name in self._prop_types:
+        #    raise TypeError
+            
         self._prop_types[prop_name] = prop_type
-        
+    
     def _add_custom_type(self, klass, type_name, prop_name, pspec):
         if not type_name in self._custom:
             custom = self._custom[type_name] = []
@@ -233,9 +315,14 @@ class PropRegistry:
         for pspec in self._list_properties(type_name):
             if pspec.name == prop_name:
                 return pspec
-
+            
+        # List child properties even if we're not specifying a
+        # parent        
         if parent:
-            for pspec in self._list_child_properties(type_name, parent):
+            type_name = parent
+
+        if gobject.type_is_a(type_name, gtk.Container):
+            for pspec in self._list_child_properties(type_name):
                 if pspec.name == prop_name:
                     return pspec
         
@@ -265,26 +352,50 @@ class PropRegistry:
         
         return pspecs
     
-    def _list_child_properties(self, type_name, parent):
-        "List all the child properties for type_name in the context of parent"
+    def _list_child_properties(self, parent):
+        "List all the child properties for parent"
         if not parent:
             return []
-        
-        gtype = gobject.type_from_name(type_name)
-        retval = []
-        parent_type = parent.__gtype__
-        widget_type = gobject.type_parent(gtk.Container)
-        type_name = gobject.type_name(parent_type)
-        for pspec in gtk.container_class_list_child_properties(type_name):
-            prop_type = self._create_type_from_pspec(pspec, child=True)
-            retval.append(prop_type)
 
-        return retval
+        return gtk.container_class_list_child_properties(parent)
 
 class PropMeta(type):
+    """
+    I am a metaclass used for Property types.
+    Currently I only offer a __repr__ method, but there
+    is also a class method called new which is used as
+    a factory for types which has me as a metaclass.
+    """
+    
     def new(meta, owner_name, name, base, value_type, owner_type,
             readable=True, writable=True, child=False, label=None):
 
+        cls = meta(owner_name + '::' + name, (base,), {})
+
+        # This can not be overridden
+        cls.name =  name
+        cls.type = value_type
+        cls.owner_name = owner_name
+        cls.owner_type = owner_type
+        
+        # Be very careful here.
+        # Do not overwrite anything, since we want this class
+        # to be as transparent as possible.
+        # Eg when doing:
+        #
+        # class Overridden(Custom, Int):
+        #   ...
+        # pr.override('some-prop', Foo)
+        #
+        # We want to have methods and attributes in the overridden to
+        # to take precedence over the one in the transparent subclass
+        # we're going to create, so check before assigning anything.
+
+        # This is clearly wrong, some parts depend on it
+        # See widgets/base/buttons: RadioGroupProp
+        cls.readable = readable
+        cls.writable = writable
+
         if not readable:
             getter = PropType._value_not_readable
         elif child:
@@ -299,20 +410,19 @@ class PropMeta(type):
         else:
             setter = base._set
 
-        cls = meta(owner_name + '::' + name, (base,), {})
-        cls.name =  name
-        cls.owner_name = owner_name
-        cls.type = value_type
-        cls.owner_type = owner_type
-        cls.value = property(getter, setter)
-        cls.readable = readable
-        cls.writable = writable
-        cls.set = setter
-        cls.get = getter
+        if cls.value is None:
+            cls.value = property(getter, setter)
+        if not cls.set:
+            cls.set = setter
+        if not cls.get:
+            cls.get = getter
         cls.child = child
 
-        # XXX: tooltip
-        cls.label = name.capitalize()
+        if not cls.label:
+            # XXX: tooltip
+            if not label:
+                label = name.capitalize()
+            cls.label = label
         
         return cls
     new = classmethod(new)
@@ -325,21 +435,42 @@ class PropMeta(type):
         
 class PropType(object):
     __metaclass__ = PropMeta
-    # name of the property, eg expand
+    """
+    I am the base class for all properties.
+    I wrap a GParamSpec (or custom property types), instances
+    of me wrap a GObject property.
+    
+    @cvar name:         name of the property, eg expand
+    @cvar owner_name:   owner gobject name: GtkWindow
+    @cvar type:         type, GType of property
+    @cvar owner_type:   type of owner
+    @cvar value:        value property
+    @cvar readable:     can the value be retreived
+    @cvar writable:     if it's possible to change values
+    @cvar set:          set method
+    @cvar get:          get method
+    @cvar child:
+    @cvar enabled:
+    @cvar translatable: marked as translatable
+    @cvar label:        property descriptor for set/get
+    @cvar default:
+    @cvar custom:
+    @cvar editable:
+    @cvar base_type:
+    @cvar editor:       editor instance, subclass of XXX
+    @cvar persistent:   Should we save the property?
+    """
+    
     name = None
-    # owner gobject name: GtkWindow
     owner_name = None
-    # type, GType of property
     type = None
-    # type of owner
-    owner_type = None
-    # value property
+    owner_type = gobject.TYPE_INVALID
     value = None
     readable = True
     writable = True
     set = None
     get = None
-    child = False
+    child = None
     enabled = True
     translatable = False
     label = None
@@ -347,58 +478,75 @@ class PropType(object):
     custom = False
     editable = True
     base_type = None
-    editor = None
+    editor = PropertyCustomEditor
+    persistent = True
+    disabled = False
+    
     def __init__(self, widget):
+        cls = type(self)
+        assert cls.name != None
+        assert cls.owner_name != None
+        assert cls.owner_type != gobject.TYPE_INVALID
+        
         self.widget = widget
         self._project = widget.project
-
-        object = widget.gtk_widget
-        # If we're a child property we're actually modifying the
-        # parent
+        gobj = widget.gtk_widget
+        self._adapter = adapter_registry.get_adapter(gobj,
+                                                     self._project.get_loader())
+        
+        # Assert that the owner is of correct type
         if self.child:
-            self._child = object
-            self._object = object.get_parent()
+            test_object = gobj.get_parent()
         else:
-            if not gobject.type_is_a(object, self.owner_type):
-                raise TypeError("Expected an object of type %s, got %r" % (
-                    gobject.type_name(self.owner_type), object))
+            test_object = gobj
+        if not gobject.type_is_a(test_object, self.owner_type):
+            raise TypeError("Expected an object of type %s, got %r" % (
+                gobject.type_name(self.owner_type), test_object))
             
-            self._child = None
-            self._object = object
-
+        self.load()
+        
+    def get_object(self):
+        return self.widget.gtk_widget
+    object = property(get_object)
+    
+    def load(self):
+        parent_name = None
         if self.child:
-            parent_name = gobject.type_name(self._object)
-        else:
-            parent_name = None
+            parent_name = gobject.type_name(self.object.get_parent())
             
         # use the gtk value if this is an internal child
         if self.widget.internal_name or self.widget.maintain_gtk_properties:
-            self._initial = widget.adaptor.get_default_prop_value(self, parent_name)                    
+            self._initial = self.widget.adaptor.get_default_prop_value(self, parent_name)
             # we have to initialize _value from the gtk widget
             if self.custom:
                 self._value = self._initial
             else:
                 if self.child:
-                    self._value = self._object.child_get_property(self._child,
-                                                                  self.name)
+                    parent = self.object.get_parent()
+                    self._value = parent.child_get_property(self.object,
+                                                            self.name)
                 else:
-                    self._value = self._object.get_property(self.name)
-                    
+                    # FIXME: This is broke, but needed for GtkRadioButton::group,
+                    # see tests.test_glade.GladeTest.test_radiobutton
+                    try:
+                        self._value = self.object.get_property(self.name)
+                    except TypeError:
+                        self._value = None
         else:
             if self.custom:
                 self._initial = self.default
                 self.set(self.default)
             else:
-                self._initial = widget.adaptor.get_default_prop_value(self, parent_name)
+                self._initial = self.widget.adaptor.get_default_prop_value(self, parent_name)
 
     def get_project(self):
         return self._project
     
     def get_object_name(self):
-        if isinstance(self._object, gtk.Widget):
-            return self._object.name
+        if isinstance(self.object, gtk.Widget):
+            return self.object.name
         
-        return repr(self._object)
+        return repr(self.object)
 
     def _value_not_readable(self):
         raise PropertyError("%s not readable" % self.name)
@@ -408,35 +556,49 @@ class PropType(object):
 
     def _child_get(self):
         try:
-            return self._object.child_get_property(self._child,
-                                                   self.name)
-        except TypeError:
-            raise TypeError('Failed to get child property from %r for %s '
-                            '(%s:%s)' % (self._object,
-                                         self._child,
-                                         gobject.type_name(self.owner_type),
-                                         self.name))
+            parent = self.object.get_parent()
+            return parent.child_get_property(self.object, self.name)
+        except TypeError, e:
+            raise TypeError("Failed to get child property from %r for %s "
+                            "(%s:%s): `%s'" % (parent,
+                                               self.object,
+                                                gobject.type_name(self.owner_type),
+                                               self.name,
+                                               e))
         
     def _child_set(self, value):
         try:
-            self._object.child_set_property(self._child,
-                                            self.name,
-                                            value)
-        except TypeError:
-            raise TypeError('Failed to set property %s:%s to %s' % (
-                gobject.type_name(self.owner_type), self.name, value))
+            parent = self.object.get_parent()
+            parent.child_set_property(self.object,
+                                      self.name,
+                                      value)
+        except TypeError, e:
+            raise TypeError("Failed to set property %s:%s to %s: `%s'" % (
+                gobject.type_name(self.owner_type), self.name, value, e))
                 
 
     def _get(self):
-        return self._object.get_property(self.name)
+        try:
+            return self.object.get_property(self.name)
+        except TypeError, e:
+            raise TypeError("Error while getting %s:%s `%s'" % (
+                gobject.type_name(self.owner_type), self.name, e))
 
     def _set(self, value):
-        self._object.set_property(self.name, value)
+        try:
+            self.object.set_property(self.name, value)
+        except TypeError, e:
+            raise TypeError("Error while setting %s:%s to %s `%s'" % (
+                gobject.type_name(self.owner_type), self.name, value, e))
 
     def get_default(cls, gobj):
         if cls.custom:
             return cls.default
         
+        if cls.child:
+            parent = gobj.get_parent()
+            return parent.child_get_property(gobj, cls.name)
+            
         return gobj.get_property(cls.name)
         
     def is_translatable(self):
@@ -461,11 +623,12 @@ class PropType(object):
         return str(value)
         
     def __repr__(self):
-        return '<Property %s value=%r>' % (self.__class__.__name__,
-                                           self.get())
+        return '<Property %s:%s value=%r>' % (self.owner_name, self.name,
+                                              self.get())
 
+# XXX: This is a really bad name since it conflicts with 
+#      the custom widget
 class CustomProperty(PropType):
-
     def get(self):
         return self._get()
 
@@ -474,8 +637,28 @@ class CustomProperty(PropType):
         
     value = property(lambda self: self.get(),
                      lambda self, value: self.set(value))
-                
-class TransparentProperty(CustomProperty):
+
+# This is of course a big hack, but it is for some reason
+# required for epydoc to run
+if environ.epydoc:
+    class FakeRegistry:
+        def add_base_type(*args, **kwargs): pass
+        def override_property(*args, **kwargs): pass
+    prop_registry = FakeRegistry()
+    custom = object
+else:
+    prop_registry = PropRegistry()
+    custom = CustomProperty
+
+class CustomChildProperty(custom):
+    child = True
+    def get(self):
+        return self._child_get()
+    
+    def set(self, value):
+        self._child_set(value)
+
+class TransparentProperty(custom):
     """
     A transparent property is a property which does not
     update the property of the GObject, instead it keeps track
@@ -492,8 +675,6 @@ class TransparentProperty(CustomProperty):
     
     def set(self, value):
         self._value = value
-        
-prop_registry = PropRegistry()
 
 class BooleanType(PropType):
     default = False
@@ -593,6 +774,9 @@ class EnumType(PropType):
     enum_class = None
     def get_choices(cls):
         choices = []
+        if not hasattr(cls.enum_class, '__enum_values__'):
+            raise UnsupportedProperty
+        
         for value, enum in cls.enum_class.__enum_values__.items():
             choices.append((enum.value_nick, enum))
         choices.sort()
@@ -611,6 +795,9 @@ prop_registry.add_base_type(EnumType, gobject.TYPE_ENUM)
 class FlagsType(PropType):
     flags_class = None
     def get_choices(cls):
+        if not hasattr(cls.flags_class, '__flags_values__'):
+            raise UnsupportedProperty
+
         flags = [(flag.first_value_name, mask)
                      for mask, flag in cls.flags_class.__flags_values__.items()
                          # we avoid composite flags
@@ -635,16 +822,35 @@ class CharType(PropType):
     default = '\0'
 prop_registry.add_base_type(CharType, gobject.TYPE_CHAR)
 
-class AdjustmentType(PropType):
+class AdjustmentType(TransparentProperty):
     def save(self):
-        value = self.get()
-        values = map(float, (value.value, value.lower, value.upper,
-                             value.step_increment, value.page_increment,
-                             value.page_size))
-        return ' '.join(values)
-        
+        adj = self.get()
+        s = [] 
+        for value in (adj.value, adj.lower, adj.upper,
+                      adj.step_increment, adj.page_increment,
+                      adj.page_size):
+            # If it's a float which ends with .0, convert to an int,
+            # so we can avoid saving unnecessary decimals
+            if str(value).endswith('.0'):
+                value = int(value)
+            s.append(str(value))
+        return ' '.join(s)
 prop_registry.add_base_type(AdjustmentType, gtk.Adjustment)
 
+class ProxiedProperty(TransparentProperty):
+    target_name = None
+    
+    def get_target(self):
+        raise NotImplementedError
+    
+    def get(self):
+        target = self.get_target()
+        return target.get_property(self.target_name)
+        
+    def set(self, value):
+        target = self.get_target()
+        return target.set_property(self.target_name, value)
+
 # TODO
 #   TYPE_INT64
 #   TYPE_LONG
@@ -653,6 +859,7 @@ prop_registry.add_base_type(AdjustmentType, gtk.Adjustment)
 #   TYPE_ULONG
 #   TYPE_UNICHAR
 
+
 if __name__ == '__main__':
     win = gtk.Window()
     box = gtk.HBox()
@@ -675,111 +882,3 @@ if __name__ == '__main__':
         
     raise SystemExit
 
-class PropertyCustomEditor(object):
-    """Base class for editors for custom properties
-
-    The create method is called the first time a widget with this property is
-    shown in the editor. Then, everytime a instance of this type of widget is
-    selected the 'update' method is called with the proper information to
-    manipulate the widget.
-
-    Because of that this should not store any information about the state of
-    the widget that is being edited.
-    """
-
-    # if wide_editor is True the editor will expand both columns of the
-    # editor page
-    wide_editor = False
-
-    def get_editor_widget(self):
-        "Return the gtk widget that implements this editor"
-        
-    def update(self, context, gtk_widget, proxy):
-        """Called when a widget with this property is selected
-
-        @param    context: gazpacho context to get the current app and project
-        @param gtk_widget: the widget we are editing
-        @param      proxy: an object that should be used to update the widget.
-                           This is what allows the undo/redo mechanism actually
-                           works
-        """
-
-class PropertyEditorDialog(gtk.Dialog):
-    """This is a base abstract class for those editors that need to be
-    opened in a dialog.
-    """
-
-    def __init__(self):
-        """The widgets of the dialog are created in the virtual method
-        _create_widgets.
-        """
-        gtk.Dialog.__init__(self,
-                            title='',
-                            parent=None,
-                            flags=gtk.DIALOG_DESTROY_WITH_PARENT,
-                            buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
-        self.widget = None
-        self.gwidget = None
-        self.proxy = None
-        
-        # avoid destrcution when clicking the x button
-        self.connect('delete-event', self._on_delete_event)
-        self.connect('response', self._on_response)
-
-        self.set_border_width(6)
-        self.set_resizable(False)
-        self.set_has_separator(False)
-        self.vbox.set_spacing(6)
-        
-        self._create_widgets()
-
-        self.vbox.show_all()
-
-    def _create_widgets(self):
-        """Subclasses should override this."""
-
-    def set_widget(self, widget, proxy):
-        """Every time the user clicks on a custom property editor button
-        we are called with a different widget and proxy.
-        """
-        from gazpacho.widget import Widget
-
-        self.widget = widget
-        self.proxy = proxy
-        self.gwidget = Widget.from_widget(widget)
-
-    def _on_delete_event(self, dialog, event):
-        self.hide()
-        return True
-    
-    def _on_response(self, dialog, response_id):
-        self.hide()
-
-class PropertyCustomEditorWithDialog(PropertyCustomEditor):
-    """A custom editor which is just a button that opens up a dialog"""
-
-    # subclasses should probably want to override the dialog_class
-    dialog_class = PropertyEditorDialog
-    button_text = _('Edit...')
-    
-    def __init__(self):
-        self.dialog = self.dialog_class()
-        self.input = gtk.Button(self.button_text)
-        self.input.connect('clicked', self.on_button_clicked)
-        
-        self.application_window = None
-        self.proxy = None
-        self.edited_widget = None
-
-    def get_editor_widget(self):
-        return self.input
-
-    def update(self, context, gtk_widget, proxy):
-        self.application_window = context.get_application_window()
-        self.edited_widget = gtk_widget
-        self.proxy = proxy
-        
-    def on_button_clicked(self, button):
-        self.dialog.set_transient_for(self.application_window)
-        self.dialog.set_widget(self.edited_widget, self.proxy)
-        self.dialog.present()
