@@ -20,18 +20,16 @@ import gobject
 import gtk
 
 from gazpacho import util
+from gazpacho.commandmanager import command_manager
 from gazpacho.properties import prop_registry, CustomProperty, \
      PropertyCustomEditor, TransparentProperty, StringType, \
      PropType
+from gazpacho.stockicons import StockIconList
 from gazpacho.widget import Widget, load_widget_from_gtk_widget
 from gazpacho.widgets.base.base import ContainerAdaptor
 
 _ = gettext.gettext
 
-(COL_STOCK_ID,
- COL_STOCK_LABEL) = range(2)
-
-
 # Adaptors for Button
 class ButtonContentsEditor(PropertyCustomEditor):
 
@@ -46,44 +44,6 @@ class ButtonContentsEditor(PropertyCustomEditor):
     def get_editor_widget(self):
         return self.input
     
-    def _create_stock_list(self):
-        "Create the list with the stock icons"
-        liststore = gtk.ListStore(str, str)
-        stocks = gtk.stock_list_ids()
-        stocks.sort()
-        for stock in stocks:
-            info = gtk.stock_lookup(stock)
-            if info is None:
-                label = "* %s" % stock
-            else:
-                label = info[1]
-            liststore.append((stock, label.replace('_', '')))
-
-        treeview = gtk.TreeView(liststore)
-        treeview.set_headers_visible(False)
-        column = gtk.TreeViewColumn()
-        cell1 = gtk.CellRendererPixbuf()
-        column.pack_start(cell1)
-        column.add_attribute(cell1, 'stock-id', COL_STOCK_ID)
-        cell2 = gtk.CellRendererText()
-        column.pack_start(cell2)
-        column.add_attribute(cell2, 'text', COL_STOCK_LABEL)
-        treeview.append_column(column)
-        treeview.set_search_column(COL_STOCK_LABEL)
-        treeview.set_enable_search(True)
-
-        self.stock_list = treeview
-
-        treeview.get_selection().connect('changed',
-                                         self._on_stock_list_selection_changed)
-
-        sw = gtk.ScrolledWindow()
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        sw.set_shadow_type(gtk.SHADOW_IN)
-        sw.add(treeview)
-
-        return sw
-
     def _create_check_entry_button(self, table, row, check_text, button_text,
                                    entry_handler, check_handler,
                                    button_handler):
@@ -150,10 +110,38 @@ class ButtonContentsEditor(PropertyCustomEditor):
         alignment = gtk.Alignment(1.0, 0.0, 1.0, 1.0)
         alignment.set_padding(0, 12, 48, 0)
         vbox = gtk.VBox()
-        vbox.pack_start(self._create_stock_list())
+
+        self.stock = StockIconList()
+        self.stock.connect('changed', self._on_stock_list_changed)
+        self.stock.set_sensitive(True)
+        vbox.pack_start(self.stock, False, False, padding=6)
+
+        hbox = gtk.HBox()
+        vbox.pack_start(hbox, False, False)
+        
         self.check_notext = gtk.CheckButton(_('No text'))
         self.check_notext.connect('toggled', self._on_check_notext_toggled)
-        vbox.pack_start(self.check_notext, False, False)
+        hbox.pack_start(self.check_notext, False, False)
+
+        label = gtk.Label(_('Icon size:'))
+        hbox.pack_start(label, False, False, 6)
+        
+        self.icon_size = gtk.ComboBox()
+        self.icon_size.set_sensitive(False)
+        self.icon_size.connect('changed', self._on_icon_size_changed)
+        cell = gtk.CellRendererText()
+        self.icon_size.pack_start(cell, True)
+        self.icon_size.add_attribute(cell, 'text', 0)
+        model = gtk.ListStore(str, object)
+        self.icon_size.set_model(model)
+        for enum in gtk.IconSize.__enum_values__.values():
+            if enum == gtk.ICON_SIZE_INVALID:
+                continue
+            iter = model.append([enum.value_nick.capitalize(), enum])
+            if enum == gtk.ICON_SIZE_BUTTON:
+                self.icon_size.set_active_iter(iter)
+        hbox.pack_start(self.icon_size, False, False)
+
         alignment.add(vbox)
         return alignment
 
@@ -211,15 +199,7 @@ class ButtonContentsEditor(PropertyCustomEditor):
         self.custom_widgets.set_sensitive(not value)
 
         if value:
-            # if there is nothing selected in the icon list select the first
-            # item
-            selection = self.stock_list.get_selection()
-            model, sel_iter = selection.get_selected()
-            if not sel_iter:
-                # this is calling set_stock as a side effect
-                selection.select_iter(model.get_iter_first())
-            else:
-                self.set_stock()
+            self.set_stock()
         else:
             self.set_custom()
 
@@ -247,13 +227,19 @@ class ButtonContentsEditor(PropertyCustomEditor):
         self.label_position.set_sensitive(value)
         self.combo_position.set_sensitive(value)
 
-    def _on_stock_list_selection_changed(self, selection):
+    def _on_stock_list_changed(self, stocklist, stock_id):
         if not self.loading:
             self.set_stock()
 
     def _on_check_notext_toggled(self, button):
         if not self.loading:
             self.set_stock()
+
+        self.icon_size.set_sensitive(button.get_active())
+        
+    def _on_icon_size_changed(self, combobox):
+        if not self.loading: 
+            self.set_stock()
         
     def _on_entry_label_changed(self, entry):
         if not self.loading:
@@ -302,17 +288,15 @@ class ButtonContentsEditor(PropertyCustomEditor):
         if not self.gwidget:
             return
         
-        model, sel_iter = self.stock_list.get_selection().get_selected()
-        if sel_iter:
-            stock_id = model[sel_iter][COL_STOCK_ID]
-        else:
-            stock_id = None
-
+        stock_id = self.stock.get_stock_id()
         notext = self.check_notext.get_active()
+        model = self.icon_size.get_model()
+        icon_size = model[self.icon_size.get_active_iter()][1]
 
-        self.command_manager.set_button_contents(self.gwidget,
-                                                 stock_id=stock_id,
-                                                 notext=notext)
+        command_manager.set_button_contents(self.gwidget,
+                                            stock_id=stock_id,
+                                            notext=notext,
+                                            icon_size=icon_size)
 
     def set_custom(self):
         if not self.gwidget:
@@ -332,10 +316,10 @@ class ButtonContentsEditor(PropertyCustomEditor):
             model = self.combo_position.get_model()
             position = model[ac_iter][1]
             
-        self.command_manager.set_button_contents(self.gwidget,
-                                                 label=label,
-                                                 image_path=image,
-                                                 position=position)
+        command_manager.set_button_contents(self.gwidget,
+                                            label=label,
+                                            image_path=image,
+                                            position=position)
     
         
     def _set_combo_active_position(self, pos):
@@ -344,17 +328,7 @@ class ButtonContentsEditor(PropertyCustomEditor):
                 self.combo_position.set_active_iter(row.iter)
                 break
 
-    def _set_stock_list_active_stock(self, stock_id):
-        selection = self.stock_list.get_selection()
-        for row in self.stock_list.get_model():
-            if row[COL_STOCK_ID] == stock_id:
-                selection.select_iter(row.iter)
-                self.stock_list.scroll_to_cell(row.path, None,
-                                               True, 0.5, 0.0)
-                break
-            
     def update(self, context, gtk_widget, proxy):
-        self.command_manager = context.get_command_manager()
         self.gwidget = Widget.from_widget(gtk_widget)
         self.application_window = context.get_application_window()
 
@@ -370,16 +344,20 @@ class ButtonContentsEditor(PropertyCustomEditor):
         self.combo_position.set_sensitive(False)
 
         (stock_id, notext, label,
-         image_path, position) = util.get_button_state(gtk_widget)
+         image_path, position, icon_size) = util.get_button_state(gtk_widget)
 
         if stock_id:
             self.stock_widgets.set_sensitive(True)
             self.custom_widgets.set_sensitive(False)
             self.radio_stock.set_active(True)
             
-            self._set_stock_list_active_stock(stock_id)
+            self.stock.set_stock_id(stock_id)
             self.check_notext.set_active(notext)
-
+            for row in self.icon_size.get_model():
+                if row[1] == icon_size:
+                    self.icon_size.set_active_iter(row.iter)
+                    break
+            self.icon_size.set_sensitive(notext)
         else:
             self.stock_widgets.set_sensitive(False)
             self.custom_widgets.set_sensitive(True)
@@ -390,7 +368,7 @@ class ButtonContentsEditor(PropertyCustomEditor):
             self.button_label.set_sensitive(True)
             if label is not None:
                 self.entry_label.set_text(label)
-
+                
             if image_path is not None:
                 self.check_image.set_active(True)
                 self.entry_image.set_sensitive(True)
@@ -418,7 +396,7 @@ class ButtonContentsAdaptor(TransparentProperty, StringType):
     editor = ButtonContentsEditor
 
     def get(self):
-        return util.get_button_state(self._object)
+        return util.get_button_state(self.object)
 
     def save(self):
         return None
@@ -432,7 +410,7 @@ class LabelAdaptor(CustomProperty, StringType):
     def is_translatable(self):
         # If a stock id is set, do not make it translatable,
         # because we don't want to translate stock-id labels!
-        stock_id = util.get_button_state(self._object)[0]
+        stock_id = util.get_button_state(self.object)[0]
         if stock_id:
             return False
 
@@ -440,25 +418,41 @@ class LabelAdaptor(CustomProperty, StringType):
 
     def save(self):
         (stock_id, notext, label,
-         image_path, position) = util.get_button_state(self._object)
+         image_path, position, icon_size) = util.get_button_state(self.object)
         if ((stock_id and notext) 
             or (not label and image_path)
             or (label and image_path)):
             return
         else:
-            
-            return self._object.get_property('label')
+            return self.object.get_property('label')
         
-prop_registry.override_property('GtkButton::label', LabelAdaptor)
+prop_registry.override_simple('GtkButton::label', LabelAdaptor)
 
 class ButtonAdaptor(ContainerAdaptor):
-    def save(self, context, gwidget):
+    def load(self, context, gtk_widget, blacklist):
+        # XXX: Copied from widget adaptor, there ought to be a
+        #      better way of saying "I do not want to any load children"
+        
+        project = context.get_project()
+        gwidget = Widget.load(gtk_widget, project, blacklist)
+        gwidget.name = gtk_widget.name
+
+        gwidget.load_signals()
+
+        # if the widget is a toplevel we need to attach the accel groups
+        # of the application
+        if gwidget.is_toplevel():
+            gwidget.setup_toplevel()
+
+        return gwidget
+
+    def save(self, context, widget):
         """Create GazpachoWidgets for the possible children of
         this widget so they are saved with the button"""
-        gtk_button = gwidget.gtk_widget
+        gtk_button = widget.gtk_widget
         
         (stock_id, notext, label,
-         image_path, position) = util.get_button_state(gtk_button)
+         image_path, position, icon_size) = util.get_button_state(gtk_button)
 
         child = gtk_button.get_child()
         project = context.get_project()
@@ -467,8 +461,15 @@ class ButtonAdaptor(ContainerAdaptor):
             or (not label and image_path) # case 2: only image
             or (label and image_path)):   # case 3: text and image
 
-            self._create_names_for_internal_widgets(child, project)
-            load_widget_from_gtk_widget(child, project)
+            # XXX: load_widget should also check so widget is not
+            #      loaded twice
+            if not Widget.from_widget(child):
+                name = child.get_data('gazpacho::object-id')
+                if not name:
+                    self._create_names_for_internal_widgets(child, project)
+                else:
+                    child.set_name(name)
+                load_widget_from_gtk_widget(child, project)
             
             # case 4: stock item with text (nothing to do)
             # case 5: only text (nothing to do)
@@ -481,20 +482,19 @@ class ButtonAdaptor(ContainerAdaptor):
         class_name = gobject.type_name(gtk_widget)
         name = project.new_widget_name(class_name)
         gtk_widget.set_name(name)
-        if isinstance(gtk_widget, gtk.Container):
-            for child in gtk_widget.get_children():
-                self._create_names_for_internal_widgets(child, project)
+        
+        if not isinstance(gtk_widget, gtk.Container):
+            return
+        
+        for child in gtk_widget.get_children():
+            self._create_names_for_internal_widgets(child, project)
 
 # GtkButton
 prop_registry.override_simple('GtkButton::use-stock', editable=False)
 #prop_registry.override_property('GtkButton::response-id', default="0", name="Response ID")
 
-# GtkToggleButton
-prop_registry.override_simple('GtkToggleButton::label', default="toggle button")
-
 # GtkCheckButton
 prop_registry.override_simple('GtkCheckButton::draw-indicator', default=True)
-prop_registry.override_simple('GtkCheckButton::label', default="toggle button")
 
 # GtkRadioButton
 class RadioGroupProp(PropType):
@@ -509,19 +509,20 @@ class RadioGroupProp(PropType):
     
     def get(self):
         group_name = ''
-        for group in self._object.get_group():
+        for group in self.object.get_group():
             if group.get_active():
                 group_name = group.get_name()
         return group_name
     
     def set(self, group):
-        if not group in self._object.get_group():
-            self._object.set_property('group', group)
+        if not group in self.object.get_group():
+            self.object.set_property('group', group)
 
     def save(self):
         value = self.get()
-        if value == self._object.get_name():
+        if value == self.object.get_name():
             return
         return value
-        
+
+# XXX: Use override_simple, but readable/writable is currently broken
 prop_registry.override_property('GtkRadioButton::group', RadioGroupProp)
